//#include "stdafx.h"
/*

The ACL initialized by this function contains no ACEs. It is empty, as opposed to being a nonexistent ACL. If an empty ACL is applied to an object, it implicitly denies all access to that object. 

To calculate the size of an ACL, add sizeof(ACL) to the size of all the ACEs to be stored in the ACL. To calculate the size of an ACE, add the size of the ACE structure, such as sizeof(ACCESS_ALLOWED_ACE), to the length of the SID associated with the ACE, and then subtract the size of the SidStart member (which is part of both the ACE structure and the SID). Use the GetSidLength function to get the length of a specified SID.

The following example shows how to calculate the size of an access-allowed ACE:

sizeof (ACCESS_ALLOWED_ACE) - sizeof (ACCESS_ALLOWED_ACE.SidStart) 
        + GetLengthSid (pAceSid);
To calculate the size of an ACL, use the following algorithm, substituting the appropriate ACE structure in the sizeof(ACE) expression:

cbAcl = sizeof (ACL);
for (i = 0 ; i < nAceCount ; i++) {
    // subtract ACE.SidStart from the size
    cbAce = sizeof (ACE) - sizeof (DWORD);
    // add this ACE's SID length
    cbAce += GetLengthSid (pAceSid[i]);
    // add the length of each ACE to the total ACL length
    cbAcl += cbAce;
}
*/

#define _WIN32_WINNT 0x0500

#define _MSC_VER 1200


#include <windows.h>
#include <stdio.h>
#include <TCHAR.H>
#include <sddl.h>
#include <Aclapi.h>
#include <Lm.h>

LPTSTR fileName = _T("e:\\nick.txt");

#pragma comment(lib, "Netapi32.lib")

bool createProcess(LPCSTR processName, LPCSTR fileName, PROCESS_INFORMATION* ppi)
{
	char buffer[MAX_PATH];
	bool result = false;	
	STARTUPINFOA si;
	ZeroMemory(ppi, sizeof(PROCESS_INFORMATION));
	ZeroMemory(&si, sizeof(STARTUPINFO));
	si.cb = sizeof(PROCESS_INFORMATION);

	sprintf(buffer, "%s %s", processName, fileName); 
	if (CreateProcessA(NULL, buffer, NULL, NULL, FALSE, CREATE_SUSPENDED, NULL, NULL, &si, ppi))
	{		
		result = true;
	}
	return result;
}

bool getSid()
{
	const int SID_Size =84;
	HANDLE pseudoHandle, realHandle;
	DWORD lengthNeeded = 0;
	PSECURITY_DESCRIPTOR psd = NULL;
	bool result = false;
	pseudoHandle = GetCurrentProcess();
	if (DuplicateHandle(pseudoHandle, pseudoHandle, pseudoHandle, &realHandle, 0, TRUE, DUPLICATE_SAME_ACCESS))
	{
		if ((psd = malloc(SID_Size))!= NULL)
		{
			lengthNeeded = SID_Size;

			if (GetKernelObjectSecurity(realHandle, DACL_SECURITY_INFORMATION, psd, lengthNeeded, &lengthNeeded))
			{
				result = true;
			}
			else
			{				
				if (GetKernelObjectSecurity(realHandle, DACL_SECURITY_INFORMATION, psd, lengthNeeded, &lengthNeeded))
				{
					result = true;
				}
				else
				{
					free(psd);
					if ((psd = malloc(lengthNeeded))!= NULL)
					{
						if (GetKernelObjectSecurity(realHandle, DACL_SECURITY_INFORMATION, psd, lengthNeeded, &lengthNeeded))
						{
							result = true;
						}
					}
				}
			}
		}
	}
	return result;
}


	
void createFile(LPCTSTR lpFileName)
{
	HANDLE fileHandle;
	SECURITY_ATTRIBUTES sa;
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.bInheritHandle = TRUE; 
	char buffer[256];
	DWORD bytesWritten;


	// The following SDDL string means
	// the data following D: represents a DACL.
	// Each ACE is encased in parentheses.
	// An 'A' is an Allow ACE. 
	// OICI means that object and container inheritance
	//    is performed.
	// GA means Generic All Access (Full Control).
	// BA uses the Builtin Administrator's group.
	PACL  pacl;
	BOOL bPresent = TRUE, bDefault = TRUE;
	DWORD bufferSizeRequested = 0;


	LPTSTR pwszSD=_T("A:(D;;GWSDRPWPFW;;;BU)");

	if (ConvertStringSecurityDescriptorToSecurityDescriptor(
	   pwszSD,
	   SDDL_REVISION_1, 
	   &(sa.lpSecurityDescriptor), 
	   NULL)) 
	{ 
		if ((fileHandle =CreateFile(fileName, GENERIC_ALL, FILE_SHARE_DELETE, &sa, CREATE_ALWAYS, FILE_ATTRIBUTE_ENCRYPTED, NULL))
			!= INVALID_HANDLE_VALUE)
		{
			for (int i=0; i<256; i++)
			{
				buffer[i]= i;
			}
			if (WriteFile(fileHandle, buffer, 256, &bytesWritten, NULL) && bytesWritten ==256)
			{
				printf("ok\n");
			}	
			CloseHandle(fileHandle);
		}			
		LocalFree(sa.lpSecurityDescriptor);
	} 
	else
	{
		int err = GetLastError();
	}
}




void create(LPCSTR fileName, LPCSTR processName)
{
	HANDLE fileHandle;
	SECURITY_ATTRIBUTES sa;
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.bInheritHandle = TRUE; 
	char buffer[256];
	DWORD bytesWritten;
	PROCESS_INFORMATION pi;

	// The following SDDL string means
	// the data following D: represents a DACL.
	// Each ACE is encased in parentheses.
	// An 'A' is an Allow ACE. 
	// OICI means that object and container inheritance
	//    is performed.
	// GA means Generic All Access (Full Control).
	// BA uses the Builtin Administrator's group.
	PACL  pacl;
	BOOL bPresent = TRUE, bDefault = TRUE;
	DWORD bufferSizeRequested = 0;
	SECURITY_DESCRIPTOR sid;
	SECURITY_INFORMATION secInfo;

	LPTSTR pwszSD=_T("D:(D;OICI;GA;;;BA)");

	if (ConvertStringSecurityDescriptorToSecurityDescriptor(
	   pwszSD,
	   SDDL_REVISION_1, 
	   &(sa.lpSecurityDescriptor), 
	   NULL)) 
	{ 
		if ((fileHandle =CreateFileA(fileName, GENERIC_ALL, FILE_SHARE_DELETE, &sa, CREATE_ALWAYS, FILE_ATTRIBUTE_ENCRYPTED, NULL))
			!= INVALID_HANDLE_VALUE)
		{
			for (int i=0; i<256; i++)
			{
				buffer[i]= i;
			}
			if (WriteFile(fileHandle, buffer, 256, &bytesWritten, NULL) && bytesWritten ==256)
			{
				printf("ok\n");
			}
			if (GetSecurityDescriptorDacl(sa.lpSecurityDescriptor, &bPresent, &pacl, &bDefault))
			{
				if (createProcess(processName, fileName, &pi))
				{
					secInfo = DACL_SECURITY_INFORMATION;
					if (GetKernelObjectSecurity(pi.hProcess, secInfo, &sid, sizeof(SECURITY_DESCRIPTOR), 
						&bufferSizeRequested) == ERROR_SUCCESS)
					{
						if (AddAccessAllowedAce(pacl, ACL_REVISION, STANDARD_RIGHTS_ALL, &sid))
						{
							ResumeThread(pi.hThread);
							WaitForSingleObject(pi.hProcess, INFINITE);


						}
						else
						{
							int err = GetLastError();
						}
					}
				}
			}
			CloseHandle(fileHandle);
		}			
		LocalFree(sa.lpSecurityDescriptor);
	} 
}

struct SecurityInfo
{
	LPTSTR accountName;
	DWORD  cbSid;
	PSID   pSid;
	DWORD  cbDomainName;
	LPTSTR wzDomainName;
	SID_NAME_USE nameUsed;
};

bool query(LPCTSTR accountName, SecurityInfo& si)
{
	bool result = false;
	memset(&si, 0, sizeof(SecurityInfo));

	if (! LookupAccountName(NULL, accountName, NULL, &si.cbSid, NULL, &si.cbDomainName, &si.nameUsed))
	{
		if ((si.pSid = (PSID)malloc(si.cbSid))!= NULL)
		{
			if ((si.wzDomainName = (LPTSTR)malloc((si.cbDomainName+1) * sizeof(TCHAR)))!= NULL)
			{
				if (LookupAccountName(NULL, accountName, si.pSid, &si.cbSid, si.wzDomainName, &si.cbDomainName, &si.nameUsed))
				{
					if ((si.accountName = (LPTSTR)malloc((_tcslen(accountName)+1) * sizeof(TCHAR))) != NULL)
					{
						_tcscpy(si.accountName, accountName);
						result = true;
					}
				}
				else
				{
					if (!result)
					{
						free(si.wzDomainName);
						free(si.pSid);
					}
				}
			}
			else
			{
				if (!result)
				{					
					free(si.pSid);
				}
			}
		}
	}
	return result;
}


bool readFilePermission(LPCTSTR lpFileName)
{
	PSECURITY_DESCRIPTOR pSD = NULL;
	DWORD lengthNeeded = 0;
	int err;
	BYTE DAcl[1024];
	PACL pAcl = (PACL)&DAcl;
	ACL_SIZE_INFORMATION aclSizeInfo;

	BOOL bPresent, bDefault;
	HANDLE procHeap;

	PACCESS_ALLOWED_ACE pAce;
	
	procHeap = GetProcessHeap();
	GetFileSecurity(lpFileName, DACL_SECURITY_INFORMATION, pSD, 0, &lengthNeeded);

	pSD = HeapAlloc(procHeap, HEAP_GENERATE_EXCEPTIONS | HEAP_ZERO_MEMORY, lengthNeeded);
	GetFileSecurity(lpFileName, DACL_SECURITY_INFORMATION, pSD, lengthNeeded, &lengthNeeded);
	if (GetSecurityDescriptorDacl(pSD, &bPresent, &pAcl, &bDefault))
	{
		GetAclInformation(pAcl, &aclSizeInfo, sizeof(ACL_SIZE_INFORMATION), AclSizeInformation);
		for (int i=0; i< aclSizeInfo.AceCount; i ++)
		{
			GetAce(pAcl, i, (LPVOID*)&pAce);
			if (pAce->Header.AceType == ACCESS_ALLOWED_ACE_TYPE)
			{
				printf("allowed\n");
			}
		}
	}
	else
	{
		err = GetLastError();
	}

	return true;
}

bool writeFilePermission(LPCTSTR lpFileName)
{
	PSECURITY_DESCRIPTOR pSD = NULL;
	DWORD lengthNeeded = 0;
	int err;
	BYTE DAcl[1024];
	PACL pAcl = (PACL)&DAcl;
	ACL_SIZE_INFORMATION aclSizeInfo;

	BOOL bPresent, bDefault;
	HANDLE procHeap;

	PACCESS_ALLOWED_ACE pAce;
	
	procHeap = GetProcessHeap();
	GetFileSecurity(lpFileName, DACL_SECURITY_INFORMATION, pSD, 0, &lengthNeeded);

	pSD = HeapAlloc(procHeap, HEAP_GENERATE_EXCEPTIONS | HEAP_ZERO_MEMORY, lengthNeeded);
	GetFileSecurity(lpFileName, DACL_SECURITY_INFORMATION, pSD, lengthNeeded, &lengthNeeded);
	if (GetSecurityDescriptorDacl(pSD, &bPresent, &pAcl, &bDefault))
	{
		GetAclInformation(pAcl, &aclSizeInfo, sizeof(ACL_SIZE_INFORMATION), AclSizeInformation);
		for (int i=0; i< aclSizeInfo.AceCount; i ++)
		{
			GetAce(pAcl, i, (LPVOID*)&pAce);
			if (pAce->Header.AceType == ACCESS_ALLOWED_ACE_TYPE)
			{
				printf("allowed\n");
			}
		}
	}
	else
	{
		err = GetLastError();
	}

	return true;
}



BOOL remove(LPTSTR lpszOwnFile)
{
	BYTE sidBuffer[100];
	PSID pSID = (PSID)&sidBuffer;
	SID_IDENTIFIER_AUTHORITY SIDAuth = SECURITY_NT_AUTHORITY;
	DWORD dwRes;

	// Try to remove the object's DACL.

	dwRes = SetNamedSecurityInfo(
		lpszOwnFile,                 // name of the object
		SE_FILE_OBJECT,              // type of object
		DACL_SECURITY_INFORMATION,   // change only the object's DACL
		NULL, NULL,                  // don't change owner or group
		NULL,                        // no DACL specified - allows everyone access
		NULL);                       // don't change SACL

	if (dwRes == ERROR_SUCCESS)
	{
		return TRUE; 
	}
	return FALSE;
}


BOOL modify(LPTSTR lpszOwnFile)
{
	HANDLE hToken; 
	BYTE sidBuffer[100];
	PSID pSID = (PSID)&sidBuffer;
	SID_IDENTIFIER_AUTHORITY SIDAuth = SECURITY_NT_AUTHORITY;
	DWORD dwRes;

	// Try to remove the object's DACL.

	dwRes = SetNamedSecurityInfo(
		lpszOwnFile,                 // name of the object
		SE_FILE_OBJECT,              // type of object
		DACL_SECURITY_INFORMATION,   // change only the object's DACL
		NULL, NULL,                  // don't change owner or group
		NULL,                        // no DACL specified - allows everyone access
		NULL);                       // don't change SACL



	// If the preceding call failed because access was denied, 
	// enable the SE_TAKE_OWNERSHIP_NAME privilege, create a SID for 
	// the Administrators group, take ownership of the object, and 
	// disable the privilege. Then try again to set the object's DACL.

	// Open a handle to the access token for the calling process.
	if (!OpenProcessToken(GetCurrentProcess(), 
						  TOKEN_ADJUST_PRIVILEGES, &hToken)) {
		  printf("OpenProcessToken failed: %u\n", GetLastError()); 
		  return FALSE; 
	} 
/*
	// Enable the SE_TAKE_OWNERSHIP_NAME privilege.
	if (!SetPrivilege(hToken, SE_TAKE_OWNERSHIP_NAME, TRUE)) {
		printf("You must be logged on as Administrator.\n");
		return FALSE; 
	}
*/
	// Create a SID for the BUILTIN\Administrators group.
	if (!AllocateAndInitializeSid(&SIDAuth, 2,
					 SECURITY_BUILTIN_DOMAIN_RID,
					 DOMAIN_ALIAS_RID_ADMINS,
					 0, 0, 0, 0, 0, 0,
					 &pSID)) {
		printf("AllocateAndInitializeSid error %u\n", GetLastError());
		return FALSE;
	}

	// Set the owner in the object's security descriptor.
	dwRes = SetNamedSecurityInfo(
		lpszOwnFile,                 // name of the object
		SE_FILE_OBJECT,              // type of object
		OWNER_SECURITY_INFORMATION,  // change only the object's owner
		pSID,                        // SID of Administrator group
		NULL, NULL, NULL); 

	if (pSID)
		FreeSid(pSID); 

	if (dwRes != ERROR_SUCCESS) {
		printf("Could not set owner. Error: %u\n", dwRes); 
		return FALSE;
	}
/*
	// Disable the SE_TAKE_OWNERSHIP_NAME privilege.
	if (!SetPrivilege(hToken, SE_TAKE_OWNERSHIP_NAME, FALSE)) {
		printf("You must be logged on as Administrator.\n");
		return FALSE; 
	}
*/
	// Try again to remove the object's DACL, now that we are the owner.
	dwRes = SetNamedSecurityInfo(
		lpszOwnFile,                 // name of the object
		SE_FILE_OBJECT,              // type of object
		DACL_SECURITY_INFORMATION,   // change only the object's DACL
		NULL, NULL,                  // don't change owner or group
		NULL,                        // no DACL specified - allows everyone access
		NULL);                       // don't change SACL 

	if (dwRes == ERROR_SUCCESS)
		return TRUE; 

	printf("Second SetNamedSecurityInfo call failed: %u\n", dwRes); 
	return FALSE;

}

void test1()
{
	SecurityInfo si;
	if (query(_T("administrator"), si))
	{
		_tprintf(_T("domain=%s, type %u\n"), si.wzDomainName, si.nameUsed);
	}
}



void test2()
{
	/*
	create(fileName);
	if (modify(fileName))
	{
		printf("what");
	}
	*/
}

int test4(LPTSTR server, LPTSTR user)
{
	NET_API_STATUS nStatus;
    nStatus = NetUserDel(server,  user);
	return nStatus;

}

int test3(int argc, wchar_t *argv[])
{
   USER_INFO_1 ui;
   DWORD dwLevel = 1;
   DWORD dwError = 0;
   NET_API_STATUS nStatus;

   if (argc != 3)
   {
      fwprintf(stderr, L"Usage: %s \\\\ServerName UserName\n", argv[0]);
      exit(1);
   }
   //
   // Set up the USER_INFO_1 structure.
   //  USER_PRIV_USER: name identifies a user, 
   //    rather than an administrator or a guest.
   //  UF_SCRIPT: required for LAN Manager 2.0 and
   //    Windows NT and later.
   //
   ui.usri1_name = argv[2];
   ui.usri1_password = argv[2];
   ui.usri1_priv = USER_PRIV_USER;
   ui.usri1_home_dir = NULL;
   ui.usri1_comment = NULL;
   ui.usri1_flags = UF_SCRIPT|UF_WORKSTATION_TRUST_ACCOUNT;
   ui.usri1_script_path = NULL;
   //
   // Call the NetUserAdd function, specifying level 1.
   //
   nStatus = NetUserAdd(argv[1],
                        dwLevel,
                        (LPBYTE)&ui,
                        &dwError);
   //
   // If the call succeeds, inform the user.
   //
   if (nStatus == NERR_Success)
      fwprintf(stderr, L"User %s has been successfully added on %s\n",
               argv[2], argv[1]);
   //
   // Otherwise, print the system error.
   //
   else
      fprintf(stderr, "A system error has occurred: %d\n", nStatus);

   return 0;
}

/*
void test5(LPCTSTR fileName)
{
	HANDLE fileHandle;
	SECURITY_INFORMATION si;	

	LookupAccountName(
	if ((fileHandle = CreateFile(fileName, GENERIC_ALL, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL))!= INVALID_HANDLE_VALUE)
	{
		ZeroMemory(&si, sizeof(SECURITY_INFORMATION));
		GetUserObjectSecurity(fileHandle, DACL_SECURITY_INFORMATION, &si, 

	}
}
*/

/*
			if (GetAce(pAcl, i, (LPVOID*)(&pAce)))
					{							
						switch (pAce->AceType)
						{
						case ACCESS_ALLOWED_ACE_TYPE:
							printf("ace_size=%u type=%u, sizeof= %u\n", pAce->AceSize, pAce->AceFlags, sizeof(ACCESS_ALLOWED_ACE));
							printf("ok\n");
							break;
						case ACCESS_ALLOWED_OBJECT_ACE_TYPE:

							break;
						case ACCESS_DENIED_ACE_TYPE:
							
							break;
						case ACCESS_DENIED_OBJECT_ACE_TYPE:

							break;
						}	
					}
					else
					{
						int err = GetLastError();
					}*/


void test6(LPTSTR fileName)
{
	PACL pAcl;
	PSECURITY_DESCRIPTOR  pSid;
	BOOL bDefault, bPresent = TRUE;
	ACL_SIZE_INFORMATION info;
	PACE_HEADER pAce = NULL;
	
	if (GetNamedSecurityInfo(fileName, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL, &pAcl, NULL, &pSid)==ERROR_SUCCESS)
	{
		if (GetSecurityDescriptorDacl(pSid, &bPresent, &pAcl, &bDefault))
		{
			if (GetAclInformation(pAcl, &info, sizeof(ACL_SIZE_INFORMATION),  AclSizeInformation))
			{
				for (int i=0; i< info.AceCount; i ++)
				{				
					if (!DeleteAce(pAcl, 0))
					{
						printf("error \n");
					}
				}
			}
		}		
		LocalFree(pSid);
	}
}

void test7()
{
		//EncryptFile(_T("e:\\password.txt"));
	LPTSTR argv[4];
	LPTSTR user = _T("nick");
	LPTSTR server = _T("\\\\nickhp");

	test4(server, user);

	argv[0]= _T("test3");
	argv[1]=server;
	argv[2]=user;
	argv[3]=NULL;
	test3(3, argv);
}

bool getProcessSecurity()
{
	HANDLE handle;
	BYTE DAcl[1024];
	PACL pAcl = (PACL)&DAcl;
	PSECURITY_DESCRIPTOR pSD = NULL;

	handle = GetCurrentProcess();
	if (GetSecurityInfo(handle, SE_KERNEL_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL, &pAcl, NULL, &pSD) == ERROR_SUCCESS)
	{
		printf("succeed\n");
	}

	return true;
}

void test10()
{
	FILE* stream;
	char buffer[256];
	stream = fopen("C:\\DOCUME~1\\ADMINI~1\\LOCALS~1\\Temp\\nick.", "r");
	if (stream != NULL)
	{
		fread(buffer, 256, 1, stream);
		printf("failed \n");
	}

}

void test11(LPSTR fileName)
{
	FILE* stream;
	char buffer[MAX_PATH];
	stream = fopen(fileName, "r");
	while (fscanf(stream, "%s", buffer)!= EOF)
	{

		printf("%s, len=%d, offset=%d\n", buffer, strlen(buffer), ftell(stream));
	}
	fclose(stream);
}

void test12()
{
	LPSTR timeStr ="Tue, 15 Nov 1994 12:45:26";
	CHAR week[7], month[7];
	int day, year, hour, minute, second;

	sscanf(timeStr,  "%s, %d %s %d %d:%d:%d", week, &day, month, &year, &hour, &minute, &second);
}

void main() 
{
	//TCHAR buffer[256];
	//GetCurrentDirectory(256, buffer);

	//LPTSTR targetFileName = _T("nickpass.txt");
	//create("nickpass.txt", "helloworld");
	//if (getSid())
	//if (readFilePermission(targetFileName))
	//test10();
	//createFile(_T("what.txt"));
	{
		//printf("ok\n");
	}
	//test11("c:\\nick");
	test12();

	//remove(_T("e:\\password.txt"));
	//create(_T("e:\\nicktest.txt"));
	//test6(_T("e:\\password.txt"));
} 
